/*
 * Copyright 1995, 1996, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
/* alpha.c - show how alpha values can be used for transparency
 *
 *  Escape key 		 	- exit the program 
 *  Left Mouse Button		- change incidence and azimuth angles
 *  Middle Mousebutton		- change the twist angle based on
 *				  horizontal mouse movement
 *  Right Mousebutton		- zoom in and out based on vertical
 *				  mouse movement
 *  <b> key			- cycle through different blend functions
 *  <R> key			- reset view
 */
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include <math.h>
#include <stdio.h>	/* for printf */

/*  Function Prototypes  */

GLvoid  initgfx( GLvoid );
GLvoid  drawScene( GLvoid );
GLvoid  reshape( GLsizei, GLsizei );
GLvoid  keyboard( GLubyte, GLint, GLint );
GLvoid  mouse( GLint, GLint, GLint, GLint );
GLvoid  motion( GLint, GLint );
GLvoid  initialize( char * );
GLvoid  blendFuncCycle( GLvoid );

void resetView( GLvoid );
void polarView( GLfloat, GLfloat, GLfloat, GLfloat);
void printHelp( char * );

/* Global Definitions */

#define KEY_ESC	27	/* ascii value for the escape key */

/* Global Variables */

static enum		actions { MOVE_EYE, TWIST_EYE, ZOOM, MOVE_NONE };
static GLint		action;

static GLint		xStart = 0, yStart = 0;

static GLfloat 		near, far, distance, twistAngle, incAngle, azimAngle;

static void     	*strokeFont;

GLvoid
main( int argc, char *argv[] )
{
	GLsizei width, height;

	glutInit( &argc, argv );

	width = glutGet( GLUT_SCREEN_WIDTH ); 
	height = glutGet( GLUT_SCREEN_HEIGHT );
	glutInitWindowPosition( 0, height / 4 );
	glutInitWindowSize( (width / 2) - 4, height / 2 );
	glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE );
	glutCreateWindow( argv[0] );

	initgfx();

	glutMouseFunc( mouse );
	glutMotionFunc( motion );
	glutKeyboardFunc( keyboard );
	glutReshapeFunc( reshape );
	glutDisplayFunc( drawScene ); 

	printHelp( argv[0] );

	glutMainLoop();
}

void
printHelp( char *progname )
{
	fprintf(stdout, "\n%s - demonstrates alpha blending\n\n"
		"Axes: X - red, Y - green, Z - blue\n\n"
		"Left Mousebutton	- move eye position\n"
		"Middle Mousebutton	- change twist angle\n"
		"Right Mousebutton	- move up / down to zoom in / out\n"
		"Escape Key		- exit the program\n"
		"<b> Key		- cycle through 3 blend functions\n"
		"<R> Key		- reset view\n",
		progname);
}

GLvoid
initgfx( GLvoid )
{
	GLfloat maxObjectSize = 3.0;

	glClearColor( 0.0, 0.0, 0.0, 0.0 );
	glShadeModel( GL_FLAT );

	/* Set up near and far so that ( far - near ) > maxObjectSize, */
	/* and determine the viewing distance (adjust for zooming) */
	near = 1.0;
	far = near + 8*maxObjectSize; 

	resetView();

	/* set the initial blend function */
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
	fprintf(stdout, 
	    "glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA )\n" );

	strokeFont = GLUT_STROKE_ROMAN;
}

GLvoid
reshape( GLsizei width, GLsizei height )
{
	GLdouble			aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLdouble) width / (GLdouble) height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, near, far );
	glMatrixMode( GL_MODELVIEW );
}

GLvoid  
blendFuncCycle( GLvoid )
{
	static int whichBlendFunc = 0;
	
	whichBlendFunc = (whichBlendFunc + 1) % 3;
	
	switch (whichBlendFunc)
	{
	case 0:
		glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
		fprintf(stdout, 
		    "glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA )\n" );
		break;
	case 1:
		glBlendFunc( GL_SRC_ALPHA, GL_ONE );
		fprintf(stdout, 
		    "glBlendFunc( GL_SRC_ALPHA, GL_ONE )\n" );
		break;
	case 2:
		glBlendFunc( GL_ONE, GL_ZERO );
		fprintf(stdout, 
		    "glBlendFunc( GL_ONE, GL_ZERO )\n" );
		break;
	default:
		break;
	}
}

GLvoid 
keyboard( GLubyte key, GLint x, GLint y )
{
	switch (key) {
	case 'b':	/* cycle through several blend funcs */
		blendFuncCycle();
		glutPostRedisplay();
		break;
	case 'R':
		resetView();
		glutPostRedisplay();
		break;
	case KEY_ESC:	/* Exit whenever the Escape key is pressed */
		exit(0);
	}
}

GLvoid 
mouse( GLint button, GLint state, GLint x, GLint y )
{
	static GLint buttons_down = 0;

	if (state == GLUT_DOWN) {
		switch (button) {
		case GLUT_LEFT_BUTTON:
			action = MOVE_EYE;
			break;
		case GLUT_MIDDLE_BUTTON:
			action = TWIST_EYE;
			break;
		case GLUT_RIGHT_BUTTON:
			action = ZOOM;
			break;
		}

		/* Update the saved mouse position */
		xStart = x;
		yStart = y;
	} else {
		if (--buttons_down == 0) 
			action = MOVE_NONE;
	}

}

GLvoid
motion( GLint x, GLint y )
{
	switch (action) {
	case MOVE_EYE:
		/* Adjust the eye position based on the mouse position */
		azimAngle += (GLdouble) (x - xStart);
		incAngle -= (GLdouble) (y - yStart);
		break;
	case TWIST_EYE:
		/* Adjust the eye twist based on the mouse position */
		twistAngle = fmodf(twistAngle+(x - xStart), 360.0);
		break;
	case ZOOM:
		/* Adjust the eye distance based on the mouse position */
		distance -= (GLdouble) (y - yStart)/10.0;
		break;
	default:
		printf("unknown action %d\n", action);
	}
	
	/* Update the stored mouse position for later use */
	xStart = x;
	yStart = y;

	glutPostRedisplay();
}

void
resetView( GLvoid )
{
	distance = near + (far - near) / 2.0;
	twistAngle = 0.0;	/* rotation of viewing volume (camera) */
	incAngle = 0.0;
	azimAngle = 0.0;
}

void
polarView( GLfloat distance, GLfloat azimuth, GLfloat incidence,
			GLfloat twist)
{
	glTranslatef( 0.0, 0.0, -distance);
	glRotatef( -twist, 0.0, 0.0, 1.0);
	glRotatef( -incidence, 1.0, 0.0, 0.0);
	glRotatef( -azimuth, 0.0, 0.0, 1.0);
}
GLvoid
renderStrokeString( void *font, char *string )
{
        int i;
        int len = (int) strlen(string);
        for (i = 0; i < len; i++) {
                glutStrokeCharacter(font, string[i]);
        }
}

void
drawFrontRectangle( GLvoid )
{
	static GLfloat red[] = { 1.0, 0.0, 0.0, 0.75 };

	glPushMatrix();
		glTranslatef( 0.0, 0.0, 0.1 );
		glColor4fv( red );
		glRectf( 0.0, 0.0, 2.0, 2.0 );
	glPopMatrix();

	glPushMatrix();
		glTranslatef( 1.0, 1.5, 0.1 );
		glScalef(0.003, 0.003, 0.003);
		glColor4f( 1.0, 1.0, 1.0, 0.75 );
		renderStrokeString( strokeFont, "Front" );
	glPopMatrix();
}

void
drawBackRectangle( GLvoid )
{
	static GLfloat blue[] = { 0.0, 0.0, 1.0, 0.75 };

	glColor4fv( blue );
	glRectf( -1.0, -1.0, 1.0, 1.0 );

	glPushMatrix();
		glTranslatef( -0.9, -0.9, 0.0 );
		glScalef(0.003, 0.003, 0.003);
		glColor4f( 1.0, 1.0, 1.0, 0.75 );
		renderStrokeString( strokeFont, "Back" );
	glPopMatrix();
}


GLvoid
drawScene( GLvoid )
{
	glClear( GL_COLOR_BUFFER_BIT );

	glPushMatrix();
		polarView( distance, azimAngle, incAngle, twistAngle );
		XYZaxes();

		/* enable blending */
		glEnable( GL_BLEND );

		/* draw partially overlapping rectangles */
		glPushMatrix ();
			glTranslatef( -3.0, 0.0, 0.0 );

			drawBackRectangle();
			drawFrontRectangle();

		glPopMatrix ();

		/* draw partially overlapping rectangles again,
		 * but reverse the drawing order 
		 */
		glPushMatrix ();
			glTranslatef( 3.0, 0.0, 0.0 );

			drawFrontRectangle();
			drawBackRectangle();

		glPopMatrix ();

		/* disable blending */
		glDisable( GL_BLEND );

	glPopMatrix();

	glutSwapBuffers();
}
