/* Copyright 1996 Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* lightIntensity.c - Use light properties to change the intensity
 *      of the light in our scene.
 *
 *  Left Mouse Button		- change incidence and azimuth angles
 *  Middle Mousebutton		- change the twist angle based on
 *				  horizontal mouse movement
 *  Right Mousebutton		- zoom in and out based on vertical
 *				  mouse movement
 *  <a> Key			- toggle ambient light intensity on/off
 *  <d> Key			- toggle diffuse light intensity on/off
 *  <s> Key			- toggle specular light intensity on/off
 *  <R> Key			- reset viewpoint
 *  Escape key			- exit the program
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include <math.h>
#include <stdio.h>

/*  Function Prototypes  */

GLvoid  initgfx( GLvoid );
GLvoid  drawScene( GLvoid );
GLvoid  reshape( GLsizei, GLsizei );
GLvoid  keyboard( GLubyte, GLint, GLint );
GLvoid  mouse( GLint, GLint, GLint, GLint );
GLvoid  motion( GLint, GLint );

GLvoid toggleAmbient( GLvoid );
GLvoid toggleDiffuse( GLvoid );
GLvoid toggleSpecular( GLvoid );

void resetView( GLvoid );
void polarView( GLfloat, GLfloat, GLfloat, GLfloat );
void printHelp( char * );

/* Global Definitions */

#define KEY_ESC	27	/* ascii value for the escape key */

/* Global Variables */

static enum		actions { MOVE_EYE, TWIST_EYE, ZOOM, MOVE_NONE };
static GLint		action;

static GLdouble		xStart = 0.0, yStart = 0.0;

static GLfloat 		fovy, near, far; 
static GLfloat		distance, twistAngle, incAngle, azimAngle;

static GLfloat 		angle = 0.0;

void
main( int argc, char *argv[] )
{
	GLsizei width, height;

	glutInit( &argc, argv );

	width = glutGet( GLUT_SCREEN_WIDTH ); 
	height = glutGet( GLUT_SCREEN_HEIGHT );
	glutInitWindowPosition( width / 4, height / 4 );
	glutInitWindowSize( (width / 2) - 4, height / 2 );
	glutInitDisplayMode( GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE );
	glutCreateWindow( argv[0] );

	initgfx();

	glutMouseFunc( mouse );
	glutMotionFunc( motion );
	glutKeyboardFunc( keyboard );
	glutReshapeFunc( reshape );
	glutDisplayFunc( drawScene ); 

	printHelp( argv[0] );

	glutMainLoop();
}

void
printHelp( char *progname )
{
	fprintf(stdout, "\n%s - demonstrate light intensity properties\n\n" 
		"Left Mousebutton	- move eye position\n"
		"Middle Mousebutton	- change twist angle\n"
		"Right Mousebutton	- move up / down to zoom in / out\n"
		"<a> Key	- toggle ambient light intensity on/off\n"
		"<d> Key	- toggle diffuse light intensity on/off\n"
		"<s> Key	- toggle specular light intensity on/off\n"
		"<R> Key		- reset viewpoint\n"
		"Escape Key		- exit the program\n\n",
		progname);
}

GLvoid
initgfx( GLvoid )
{
	glClearColor( 0.0, 0.0, 0.0, 1.0 );
	glEnable( GL_DEPTH_TEST );

	fovy = 60.0;	/* field of view in Y */
	near = 3.0;	/* Near clipping plane location */
	far  = 12.0;	/* Far clipping plane location */
	resetView();

	/* Turn on a default light */
	glEnable( GL_LIGHT0 );
}

GLvoid
toggleAmbient( GLvoid )
{
	/* Set up intensities for the light */
	static GLfloat    ambientIntensity[] = { 0.2, 0.0, 0.2, 1.0 };
	static GLfloat    defaultAmbient[] = { 0.0, 0.0, 0.0, 1.0 };

	static GLboolean useDefault = GL_TRUE;

	useDefault = !useDefault;

	if (useDefault)
		glLightfv( GL_LIGHT0, GL_AMBIENT, defaultAmbient );
	else 
		glLightfv( GL_LIGHT0, GL_AMBIENT, ambientIntensity );
}

GLvoid
toggleDiffuse( GLvoid )
{
	static GLfloat    diffuseIntensity[] = { 0.0, 0.1, 1.0, 1.0 };
	static GLfloat    defaultDiffuse[] = { 1.0, 1.0, 1.0, 1.0 };

	static GLboolean useDefault = GL_TRUE;

	useDefault = !useDefault;
	if (useDefault)
		glLightfv( GL_LIGHT0, GL_DIFFUSE, defaultDiffuse );
	else 
		glLightfv( GL_LIGHT0, GL_DIFFUSE, diffuseIntensity );
}

GLvoid
toggleSpecular( GLvoid )
{
	static GLfloat    specularIntensity[] = { 1.0, 0.0, 1.0, 1.0 };
	static GLfloat    defaultSpecular[] = { 1.0, 1.0, 1.0, 1.0 };

	static GLboolean useDefault = GL_TRUE;

	useDefault = !useDefault;
	if (useDefault)
		glLightfv( GL_LIGHT0, GL_SPECULAR, defaultSpecular );
	else 
		glLightfv( GL_LIGHT0, GL_SPECULAR, specularIntensity );
}

GLvoid 
keyboard( GLubyte key, GLint x, GLint y )
{
	switch (key) {
	case 'a':	/* toggle ambient intensity */
		toggleAmbient();
		break;
	case 'd':	/* toggle Diffuse intensity */
		toggleDiffuse();
		break;
	case 's':	/* toggle Specular intensity */
		toggleSpecular();
		break;
	case 'R':
		resetView();
		glutPostRedisplay();
		break;
	case KEY_ESC:	/* Exit whenever the Escape key is pressed */
		exit(0);
	}
	glutPostRedisplay();
}

GLvoid 
mouse( GLint button, GLint state, GLint x, GLint y )
{
	static GLint buttons_down = 0;

	if (state == GLUT_DOWN) {
		switch (button) {
		case GLUT_LEFT_BUTTON:
			action = MOVE_EYE;
			break;
		case GLUT_MIDDLE_BUTTON:
			action = TWIST_EYE;
			break;
		case GLUT_RIGHT_BUTTON:
			action = ZOOM;
			break;
		}

		/* Update the saved mouse position */
		xStart = x;
		yStart = y;
	} else {
		if (--buttons_down == 0) 
			action = MOVE_NONE;
	}
}

GLvoid
motion( GLint x, GLint y )
{
	switch (action) {
	case MOVE_EYE:
		/* Adjust the eye position based on the mouse position */
		azimAngle += (GLdouble) (x - xStart);
		incAngle -= (GLdouble) (y - yStart);
		break;
	case TWIST_EYE:
		/* Adjust the eye twist based on the mouse position */
		twistAngle = fmodf(twistAngle+(x - xStart), 360.0);
		break;
	case ZOOM:
		/* Adjust the eye distance based on the mouse position */
		distance -= (GLdouble) (y - yStart)/10.0;
		break;
	default:
		printf("unknown action %d\n", action);
	}
	
	/* Update the stored mouse position for later use */
	xStart = x;
	yStart = y;

	glutPostRedisplay();

}

void
resetView( GLvoid )
{
	distance = near + (far - near) / 2.0;
	twistAngle = 0.0;	/* rotation of viewing volume (camera) */
	incAngle = 60.0;
	azimAngle = 0.0;
	fovy = 60.0;	/* Field of view in Y angle */
}

GLvoid
reshape( GLsizei width, GLsizei height )
{
	GLdouble	aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLdouble) width / (GLdouble) height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( fovy, aspect, near, far );
	glMatrixMode( GL_MODELVIEW );
}

void
polarView( GLfloat distance, GLfloat azimuth, GLfloat incidence,
			GLfloat twist)
{
	glTranslatef( 0.0, 0.0, -distance);
	glRotatef( -twist, 0.0, 0.0, 1.0);
	glRotatef( -incidence, 1.0, 0.0, 0.0);
	glRotatef( -azimuth, 0.0, 0.0, 1.0);
}

GLvoid
drawScene( GLvoid )
{
	/* Define a few materials properties */
	GLfloat   redAmbient[] = { 0.3, 0.1, 0.1, 1.0 };
	GLfloat   redDiffuse[] = { 1.0, 0.0, 0.0, 1.0 };
	GLfloat   blueAmbient[] = { 0.1, 0.1, 0.3, 1.0 };
	GLfloat   blueDiffuse[] = { 0.0, 0.0, 1.0, 1.0 };
	GLfloat   yellowDiffuse[] = { 1.0, 1.0, 0.0, 1.0 };
	GLfloat   yellowEmission[] = { 0.6, 0.6, 0.0, 1.0 };
	GLfloat   defaultEmission[] = { 0.0, 0.0, 0.0, 1.0 };
	GLfloat   whiteSpecular[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat   greenSpecular[] = { 0.0, 1.0, 0.0, 1.0 };
	GLfloat   defaultSpecular[] = { 0.0, 0.0, 0.0, 1.0 };

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glPushMatrix();
		polarView( distance, azimAngle, incAngle, twistAngle );
		XYZaxes();

		glEnable( GL_LIGHTING );

		glMaterialfv( GL_FRONT, GL_EMISSION, defaultEmission );

		/* Set properties for a shiny red material,
		 * with a green highlight */
		glMaterialfv( GL_FRONT, GL_AMBIENT, redAmbient );
		glMaterialfv( GL_FRONT, GL_DIFFUSE, redDiffuse );
		glMaterialfv( GL_FRONT, GL_SPECULAR, greenSpecular );
		glMaterialf( GL_FRONT, GL_SHININESS, 128.0 );
		glPushMatrix();
			glTranslatef( -2.0, 1.5, 0.0 );
			glutSolidSphere( 0.7, 31, 31 );
		glPopMatrix();

		/* Set properties for a dull blue material with
		 *   a small white highlight */
		glMaterialfv( GL_FRONT, GL_AMBIENT, blueAmbient );
		glMaterialfv( GL_FRONT, GL_DIFFUSE, blueDiffuse );
		glMaterialfv( GL_FRONT, GL_SPECULAR, whiteSpecular );
		glMaterialf( GL_FRONT, GL_SHININESS, 20.0 );
		glPushMatrix();
			glTranslatef( 2.5, 0.0, 0.0 );
			glutSolidTorus( 0.25, 0.75, 16, 31 );
		glPopMatrix();

		/* Set properties for a yellow glowing material */
		glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, yellowDiffuse);
		glMaterialfv( GL_FRONT, GL_EMISSION, yellowEmission );
		glMaterialfv( GL_FRONT, GL_SPECULAR, defaultSpecular );

		glPushMatrix();
			glTranslatef( 0.0, 2.0, 2.0 );
			glutSolidCube( 0.5 );
		glPopMatrix();

		glDisable( GL_LIGHTING );

	glPopMatrix();

	glutSwapBuffers();
}
